MODULE asmphyto2dbal_ersem
   !!======================================================================
   !!                       ***  MODULE asmphyto2dbal_ersem  ***
   !! Calculate increments to ERSEM based on surface phyto2d increments
   !!
   !! IMPORTANT NOTE: This calls the bioanalysis routine of Hemmings et al.
   !! For licensing reasons this is kept in its own internal Met Office
   !! branch (dev/frdf/vn3.6_nitrogen_balancing) rather than in the Paris
   !! repository, and must be merged in when building.
   !!
   !!======================================================================
   !! History :  3.6  ! 2019-01 (D. Ford)  Adapted from asmphyto2dbal_medusa
   !!----------------------------------------------------------------------
#if defined key_asminc && defined key_fabm
   !!----------------------------------------------------------------------
   !! 'key_asminc'          : assimilation increment interface
   !! 'key_fabm'            : FABM-ERSEM model
   !!----------------------------------------------------------------------
   !! asm_phyto2d_bal_ersem : routine to calculate increments to ERSEM
   !!----------------------------------------------------------------------
   USE par_kind,      ONLY: wp             ! kind parameters
   USE par_oce,       ONLY: jpi, jpj, jpk  ! domain array sizes
   USE dom_oce,       ONLY: gdepw_n        ! domain information
   USE iom                                 ! i/o
   USE par_fabm                            ! FABM-ERSEM parameters
   USE par_trc,       ONLY: jptra          ! Tracer parameters
   USE bioanalysis                         ! Nitrogen balancing

   IMPLICIT NONE
   PRIVATE                   

   PUBLIC asm_phyto2d_bal_ersem

   ! Default values for biological assimilation parameters
   ! Should match Hemmings et al. (2008)
   REAL(wp), PARAMETER :: balnutext  =  0.6    !: Default nutrient balancing factor
   REAL(wp), PARAMETER :: balnutmin  =  0.1    !: Fraction of phytoplankton loss to nutrient
   REAL(wp), PARAMETER :: r          =  1      !: Reliability of model specific growth rate
   REAL(wp), PARAMETER :: beta_g     =  0.05   !: Low rate bias correction for growth rate estimator
   REAL(wp), PARAMETER :: beta_l     =  0.05   !: Low rate bias correction for primary loss rate estimator
   REAL(wp), PARAMETER :: beta_m     =  0.05   !: Low rate bias correction for secondary loss rate estimator
   REAL(wp), PARAMETER :: a_g        =  0.2    !: Error s.d. for log10 of growth rate estimator
   REAL(wp), PARAMETER :: a_l        =  0.4    !: Error s.d. for log10 of primary loss rate estimator
   REAL(wp), PARAMETER :: a_m        =  0.7    !: Error s.d. for log10 of secondary loss rate estimator
   REAL(wp), PARAMETER :: zfracb0    =  0.7    !: Base zooplankton fraction of loss to Z & D
   REAL(wp), PARAMETER :: zfracb1    =  0      !: Phytoplankton sensitivity of zooplankton fraction
   REAL(wp), PARAMETER :: qrfmax     =  1.1    !: Maximum nutrient limitation reduction factor
   REAL(wp), PARAMETER :: qafmax     =  1.1    !: Maximum nutrient limitation amplification factor
   REAL(wp), PARAMETER :: zrfmax     =  2      !: Maximum zooplankton reduction factor
   REAL(wp), PARAMETER :: zafmax     =  2      !: Maximum zooplankton amplification factor
   REAL(wp), PARAMETER :: prfmax     =  10     !: Maximum phytoplankton reduction factor (secondary)
   REAL(wp), PARAMETER :: incphymin  =  0.0001 !: Minimum size of non-zero phytoplankton increment
   REAL(wp), PARAMETER :: integnstep =  20     !: Number of steps for p.d.f. integral evaluation
   REAL(wp), PARAMETER :: pthreshold =  0.01   !: Fractional threshold level for setting p.d.f.
   !
   LOGICAL,  PARAMETER :: diag_active           = .TRUE.  !: Depth-independent diagnostics
   LOGICAL,  PARAMETER :: diag_fulldepth_active = .TRUE.  !: Full-depth diagnostics
   LOGICAL,  PARAMETER :: gl_active             = .TRUE.  !: Growth/loss-based balancing
   LOGICAL,  PARAMETER :: nbal_active           = .TRUE.  !: Nitrogen balancing
   LOGICAL,  PARAMETER :: subsurf_active        = .TRUE.  !: Increments below MLD
   LOGICAL,  PARAMETER :: deepneg_active        = .FALSE. !: Negative primary increments below MLD
   LOGICAL,  PARAMETER :: deeppos_active        = .FALSE. !: Positive primary increments below MLD
   LOGICAL,  PARAMETER :: nutprof_active        = .TRUE.  !: Secondary increments

CONTAINS

   SUBROUTINE asm_phyto2d_bal_ersem(  ld_chltot,                      &
      &                               pinc_chltot,                    &
      &                               ld_chldia,                      &
      &                               pinc_chldia,                    &
      &                               ld_chlnan,                      &
      &                               pinc_chlnan,                    &
      &                               ld_chlpic,                      &
      &                               pinc_chlpic,                    &
      &                               ld_chldin,                      &
      &                               pinc_chldin,                    &
      &                               pincper,                        &
      &                               p_maxchlinc, ld_phytobal, pmld, &
      &                               pgrow_avg_bkg, ploss_avg_bkg,   &
      &                               phyt_avg_bkg, mld_max_bkg,      &
      &                               totalk_bkg,                     &
      &                               tracer_bkg, phyto2d_balinc )
      !!---------------------------------------------------------------------------
      !!                    ***  ROUTINE asm_phyto2d_bal_ersem  ***
      !!
      !! ** Purpose :   calculate increments to ERSEM from 2d phytoplankton increments
      !!
      !! ** Method  :   EITHER (ld_phytobal == .TRUE.):
      !!                average up ERSEM to look like HadOCC
      !!                call nitrogen balancing scheme
      !!                separate back out to MEDUSA
      !!                OR (ld_phytobal == .FALSE.):
      !!                calculate increments to maintain background stoichiometry
      !!
      !! ** Action  :   populate phyto2d_balinc
      !!
      !! References :   Hemmings et al., 2008, J. Mar. Res.
      !!                Ford et al., 2012, Ocean Sci.
      !!                Skakala et al., 2018, JGR
      !!---------------------------------------------------------------------------
      !!
      LOGICAL,  INTENT(in   )                               :: ld_chltot        ! Assim chltot y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj)           :: pinc_chltot      ! chltot increments
      LOGICAL,  INTENT(in   )                               :: ld_chldia        ! Assim chldia y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj)           :: pinc_chldia      ! chldia increments
      LOGICAL,  INTENT(in   )                               :: ld_chlnan        ! Assim chlnan y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj)           :: pinc_chlnan      ! chlnan increments
      LOGICAL,  INTENT(in   )                               :: ld_chlpic        ! Assim chlpic y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj)           :: pinc_chlpic      ! chlpic increments
      LOGICAL,  INTENT(in   )                               :: ld_chldin        ! Assim chldin y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj)           :: pinc_chldin      ! chldin increments
      REAL(wp), INTENT(in   )                               :: pincper          ! Assimilation period
      REAL(wp), INTENT(in   )                               :: p_maxchlinc      ! Max chl increment
      LOGICAL,  INTENT(in   )                               :: ld_phytobal      ! Balancing y/n
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj)           :: pmld             ! Mixed layer depth
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj)           :: pgrow_avg_bkg    ! Avg phyto growth
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj)           :: ploss_avg_bkg    ! Avg phyto loss
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj)           :: phyt_avg_bkg     ! Avg phyto
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj)           :: mld_max_bkg      ! Max MLD
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,jpk)       :: totalk_bkg       ! Total alkalinity
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,jpk,jptra) :: tracer_bkg       ! State variables
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk,jptra) :: phyto2d_balinc   ! Balancing increments
      !!
      INTEGER                                               :: ji, jj, jk, jn   ! Loop counters
      INTEGER                                               :: jkmax            ! Loop index
      INTEGER,                 DIMENSION(6)                 :: i_tracer         ! Tracer indices
      REAL(wp)                                              :: zmassc           ! Carbon molar mass
      REAL(wp)                                              :: zmassn           ! Nitrogen molar mass
      REAL(wp)                                              :: z4qnc            ! Z4/qnc (mesozoo N:C)
      REAL(wp)                                              :: n2be_p           ! N:biomass for total phy
      REAL(wp)                                              :: n2be_z           ! N:biomass for total zoo
      REAL(wp)                                              :: n2be_d           ! N:biomass for detritus
      REAL(wp)                                              :: zfrac            ! Fractions
      REAL(wp)                                              :: zfrac_chl1       ! 
      REAL(wp)                                              :: zfrac_chl2       ! 
      REAL(wp)                                              :: zfrac_chl3       ! 
      REAL(wp)                                              :: zfrac_chl4       ! 
      REAL(wp)                                              :: zfrac_p1n        ! 
      REAL(wp)                                              :: zfrac_p2n        ! 
      REAL(wp)                                              :: zfrac_p3n        ! 
      REAL(wp)                                              :: zfrac_p4n        ! 
      REAL(wp)                                              :: zfrac_z4n        ! 
      REAL(wp)                                              :: zfrac_z5n        ! 
      REAL(wp)                                              :: zfrac_z6n        ! 
      REAL(wp)                                              :: zfrac_n3n        ! 
      REAL(wp)                                              :: zfrac_n4n        ! 
      REAL(wp)                                              :: zfrac_r4n        ! 
      REAL(wp)                                              :: zfrac_r6n        ! 
      REAL(wp)                                              :: zfrac_r8n        ! 
      REAL(wp)                                              :: zrat_chl1_p1n    ! Ratios
      REAL(wp)                                              :: zrat_p1c_p1n     ! 
      REAL(wp)                                              :: zrat_p1p_p1n     ! 
      REAL(wp)                                              :: zrat_p1s_p1n     ! 
      REAL(wp)                                              :: zrat_chl2_p2n    ! 
      REAL(wp)                                              :: zrat_p2c_p2n     ! 
      REAL(wp)                                              :: zrat_p2p_p2n     ! 
      REAL(wp)                                              :: zrat_chl3_p3n    ! 
      REAL(wp)                                              :: zrat_p3c_p3n     ! 
      REAL(wp)                                              :: zrat_p3p_p3n     ! 
      REAL(wp)                                              :: zrat_chl4_p4n    ! 
      REAL(wp)                                              :: zrat_p4c_p4n     ! 
      REAL(wp)                                              :: zrat_p4p_p4n     ! 
      REAL(wp)                                              :: zrat_z4c_z4n     ! 
      REAL(wp)                                              :: zrat_z5c_z5n     ! 
      REAL(wp)                                              :: zrat_z5p_z5n     ! 
      REAL(wp)                                              :: zrat_z6c_z6n     ! 
      REAL(wp)                                              :: zrat_z6p_z6n     ! 
      REAL(wp)                                              :: zrat_r4c_r4n     ! 
      REAL(wp)                                              :: zrat_r4p_r4n     ! 
      REAL(wp)                                              :: zrat_r6c_r6n     ! 
      REAL(wp)                                              :: zrat_r6p_r6n     ! 
      REAL(wp)                                              :: zrat_r6s_r6n     ! 
      REAL(wp)                                              :: zrat_r8c_r8n     ! 
      REAL(wp)                                              :: zrat_r8p_r8n     ! 
      REAL(wp)                                              :: zrat_r8s_r8n     ! 
      REAL(wp)                                              :: zrat_p1c_chl1    ! 
      REAL(wp)                                              :: zrat_p1n_chl1    ! 
      REAL(wp)                                              :: zrat_p1p_chl1    ! 
      REAL(wp)                                              :: zrat_p1s_chl1    ! 
      REAL(wp)                                              :: zrat_p2c_chl2    ! 
      REAL(wp)                                              :: zrat_p2n_chl2    ! 
      REAL(wp)                                              :: zrat_p2p_chl2    ! 
      REAL(wp)                                              :: zrat_p3c_chl3    ! 
      REAL(wp)                                              :: zrat_p3n_chl3    ! 
      REAL(wp)                                              :: zrat_p3p_chl3    ! 
      REAL(wp)                                              :: zrat_p4c_chl4    ! 
      REAL(wp)                                              :: zrat_p4n_chl4    ! 
      REAL(wp)                                              :: zrat_p4p_chl4    ! 
      REAL(wp),                DIMENSION(jpi,jpj)           :: cchl_p           ! C:Chl for total phy
      REAL(wp),                DIMENSION(16)                :: modparm          ! Model parameters
      REAL(wp),                DIMENSION(20)                :: assimparm        ! Assimilation parameters
      REAL(wp),                DIMENSION(jpi,jpj,jpk,6)     :: bstate           ! Background state
      REAL(wp),                DIMENSION(jpi,jpj,jpk,6)     :: outincs          ! Balancing increments
      REAL(wp),                DIMENSION(jpi,jpj,22)        :: diag             ! Depth-indep diagnostics
      REAL(wp),                DIMENSION(jpi,jpj,jpk,22)    :: diag_fulldepth   ! Full-depth diagnostics
      REAL(wp),                DIMENSION(jpi,jpj)           :: pinc_chltot_temp ! Temporary array
      INTEGER,                 DIMENSION(1)                 :: zkmt             ! No. sea points in column
      !!---------------------------------------------------------------------------
      
      ! Set parameters, firstly molar mass of carbon and nitrogen
      zmassc = 12.01
      zmassn = 14.01
      
      ! Then mesozooplankton nitrogen(mmol):carbon(mg), ERSEM parameter Z4/qnc
      ! Hardwire it for now due to difficulty of getting at parameters through FABM
      ! I think the following should work:
      ! z4qnc = model%state_variables(jp_fabm_z4c)%properties%get_property_by_name('qnc')%value
      ! but, get_property_by_name is in the module fabm_properties, which we can't get at directly
      ! We can do a "USE fabm" which itself does a "USE fabm_properties", but doing it indirectly
      ! like that means get_property_by_name is treated as private so we can't use it
      ! So hardwire to value used in SSB runs and v4/5 CMEMS reanalysis
      z4qnc  = 0.0126

      ! If p_maxchlinc > 0 then cap total absolute chlorophyll increment at that value
      IF ( p_maxchlinc > 0.0 ) THEN
         IF ( ld_chltot ) THEN
            DO jj = 1, jpj
               DO ji = 1, jpi
                  pinc_chltot(ji,jj) = MAX( -1.0 * p_maxchlinc, MIN( pinc_chltot(ji,jj), p_maxchlinc ) )
               END DO
            END DO
         ELSE
            DO jj = 1, jpj
               DO ji = 1, jpi
                  IF ( ld_chldia .AND. ld_chlnan .AND. ld_chlpic .AND. ld_chldin ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chldia(ji,jj) + pinc_chlnan(ji,jj) + &
                        &                      pinc_chlpic(ji,jj) + pinc_chldin(ji,jj)
                  ELSE IF ( ld_chldia .AND. ld_chlnan .AND. ld_chlpic ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chldia(ji,jj) + pinc_chlnan(ji,jj) + &
                        &                      pinc_chlpic(ji,jj)
                  ELSE IF ( ld_chldia .AND. ld_chlnan .AND. ld_chldin ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chldia(ji,jj) + pinc_chlnan(ji,jj) + &
                        &                      pinc_chldin(ji,jj)
                  ELSE IF ( ld_chldia .AND. ld_chlpic .AND. ld_chldin ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chldia(ji,jj) + &
                        &                      pinc_chlpic(ji,jj) + pinc_chldin(ji,jj)
                  ELSE IF ( ld_chlnan .AND. ld_chlpic .AND. ld_chldin ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chlnan(ji,jj) + &
                        &                      pinc_chlpic(ji,jj) + pinc_chldin(ji,jj)
                  ELSE IF ( ld_chldia .AND. ld_chlnan ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chldia(ji,jj) + pinc_chlnan(ji,jj)
                  ELSE IF ( ld_chldia .AND. ld_chlpic ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chldia(ji,jj) + pinc_chlpic(ji,jj)
                  ELSE IF ( ld_chldia .AND. ld_chldin ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chldia(ji,jj) + pinc_chldin(ji,jj)
                  ELSE IF ( ld_chlnan .AND. ld_chlpic ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chlnan(ji,jj) + pinc_chlpic(ji,jj)
                  ELSE IF ( ld_chlnan .AND. ld_chldin ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chlnan(ji,jj) + pinc_chldin(ji,jj)
                  ELSE IF ( ld_chlpic .AND. ld_chldin ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chlpic(ji,jj) + pinc_chldin(ji,jj)
                  ELSE IF ( ld_chldia ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chldia(ji,jj)
                  ELSE IF ( ld_chlnan ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chlnan(ji,jj)
                  ELSE IF ( ld_chlpic ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chlpic(ji,jj)
                  ELSE IF ( ld_chldin ) THEN
                     pinc_chltot_temp(ji,jj) = pinc_chldin(ji,jj)
                  ENDIF
                  pinc_chltot(ji,jj) = MAX( -1.0 * p_maxchlinc, MIN( pinc_chltot_temp(ji,jj), p_maxchlinc ) )
                  IF ( pinc_chltot(ji,jj) .NE. pinc_chltot_temp(ji,jj) ) THEN
                     zfrac = pinc_chltot(ji,jj) / pinc_chltot_temp(ji,jj)
                     IF ( ld_chldia ) THEN
                        pinc_chldia(ji,jj) = pinc_chldia(ji,jj) * zfrac
                     ENDIF
                     IF ( ld_chlnan ) THEN
                        pinc_chlnan(ji,jj) = pinc_chlnan(ji,jj) * zfrac
                     ENDIF
                     IF ( ld_chlpic ) THEN
                        pinc_chlpic(ji,jj) = pinc_chlpic(ji,jj) * zfrac
                     ENDIF
                     IF ( ld_chldin ) THEN
                        pinc_chldin(ji,jj) = pinc_chldin(ji,jj) * zfrac
                     ENDIF
                  ENDIF
               END DO
            END DO
         ENDIF
      ENDIF
      
      ! Initialise balancing increments
      phyto2d_balinc(:,:,:,:) = 0.0
      
      IF ( ld_phytobal ) THEN   ! Nitrogen balancing

         ! Set up model parameters to be passed into Hemmings balancing routine.
         ! For now these are hardwired to the standard HadOCC parameter values
         ! as this is what the scheme was developed for.
         ! Obviously, HadOCC and ERSEM are rather different models, so this
         ! isn't ideal, but there's not always direct analogues between the two
         ! parameter sets, so it's the easiest way to get something running.
         ! In the longer term, some serious MarMOT-based development is required.
         modparm(1)  = 0.1                               ! grow_sat
         modparm(2)  = 2.0                               ! psmax
         modparm(3)  = 0.845                             ! par
         modparm(4)  = 0.02                              ! alpha
         modparm(5)  = 0.05                              ! resp_rate
         modparm(6)  = 0.05                              ! pmort_rate
         modparm(7)  = 0.01                              ! phyto_min
         modparm(8)  = 0.05                              ! z_mort_1
         modparm(9)  = 1.0                               ! z_mort_2
         !modparm(10) = 6.625                            ! c2n_p - set later per grid point
         !modparm(11) = 5.625                            ! c2n_z - set later per grid point
         !modparm(12) = 7.5                              ! c2n_d - set later per grid point
         modparm(13) = 0.01                              ! graze_threshold
         modparm(14) = 2.0                               ! holling_coef
         modparm(15) = 0.5                               ! graze_sat
         modparm(16) = 2.0                               ! graze_max

         ! Set up assimilation parameters to be passed into balancing routine
         ! Not sure what assimparm(1) is meant to be, but it doesn't get used
         assimparm(2)  = balnutext
         assimparm(3)  = balnutmin
         assimparm(4)  = r
         assimparm(5)  = beta_g
         assimparm(6)  = beta_l
         assimparm(7)  = beta_m
         assimparm(8)  = a_g
         assimparm(9)  = a_l
         assimparm(10) = a_m
         assimparm(11) = zfracb0
         assimparm(12) = zfracb1
         assimparm(13) = qrfmax
         assimparm(14) = qafmax
         assimparm(15) = zrfmax
         assimparm(16) = zafmax
         assimparm(17) = prfmax
         assimparm(18) = incphymin
         assimparm(19) = integnstep
         assimparm(20) = pthreshold

         ! Set up external tracer indices array bstate
         i_tracer(1) = 1   ! nutrient
         i_tracer(2) = 2   ! phytoplankton
         i_tracer(3) = 3   ! zooplankton
         i_tracer(4) = 4   ! detritus
         i_tracer(5) = 5   ! DIC
         i_tracer(6) = 6   ! Alkalinity

         ! Set background state
         bstate(:,:,:,i_tracer(1)) = tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_n3n) + &
            &                        tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_n4n)
         bstate(:,:,:,i_tracer(2)) = tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_p1n) + &
            &                        tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_p2n) + &
            &                        tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_p3n) + &
            &                        tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_p4n)
         bstate(:,:,:,i_tracer(3)) = (tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_z4c) * z4qnc) + &
            &                        tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_z5n) + &
            &                        tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_z6n)
         bstate(:,:,:,i_tracer(4)) = tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_r4n) + &
            &                        tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_r6n) + &
            &                        tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_r8n)
         bstate(:,:,:,i_tracer(5)) = tracer_bkg(:,:,:,jp_fabm_m1+jp_fabm_o3c)
         bstate(:,:,:,i_tracer(6)) = totalk_bkg(:,:,:)

         ! Calculate carbon to chlorophyll ratio for combined phytoplankton
         ! and nitrogen to biomass equivalent for PZD (hardwire as per HadOCC)
         cchl_p(:,:) = 0.0
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) + tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) + &
                  &   tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) + tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) ) .GT. 0.0 ) THEN
                  cchl_p(ji,jj) = zmassc * ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p1c) +      &
                     &                       tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p2c) +      &
                     &                       tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p3c) +      &
                     &                       tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p4c)   ) /  &
                     &            ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) +             &
                     &              tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) +             &
                     &              tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) +             &
                     &              tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl4)   )
               ENDIF
            END DO
         END DO

         ! Call nitrogen balancing routine - loop over grid points due to variable C:N ratios
         DO jj = 1, jpj
            DO ji = 1, jpi
               ! Phytoplankton C:N
               modparm(10) = ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p1c) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p2c) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p3c) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p4c)     ) / &
                  &          ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p1n) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p2n) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p3n) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p4n)     )
               ! Zooplankton C:N
               modparm(11) = ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_z4c) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_z5c) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_z6c)     ) / &
                  &          ( (tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_z4c) * z4qnc) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_z5n) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_z6n)     )
               ! Detritus C:N
               modparm(12) = ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_r4c) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_r6c) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_r8c)     ) / &
                  &          ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_r4n) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_r6n) + &
                  &            tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_r8n)     )
               ! Nitrogen to biomass equivalent for PZD
               n2be_p = zmassn + ( zmassc * modparm(10) )
               n2be_z = zmassn + ( zmassc * modparm(11) )
               n2be_d = zmassn + ( zmassc * modparm(12) )
               zkmt(:) = INT(SUM(tmask(ji,jj,:)))
               CALL bio_analysis( 1, 1, jpk, gdepw_n(ji,jj,2:jpk), i_tracer, modparm,                 &
                  &               n2be_p, n2be_z, n2be_d, assimparm,                                  &
                  &               INT(pincper), 1, zkmt, tmask(ji,jj,:),                              &
                  &               pmld(ji,jj), mld_max_bkg(ji,jj), pinc_chltot(ji,jj), cchl_p(ji,jj), &
                  &               nbal_active, phyt_avg_bkg(ji,jj),                                   &
                  &               gl_active, pgrow_avg_bkg(ji,jj), ploss_avg_bkg(ji,jj),              &
                  &               subsurf_active, deepneg_active,                                     &
                  &               deeppos_active, nutprof_active,                                     &
                  &               bstate(ji,jj,:,:), outincs(ji,jj,:,:),                              &
                  &               diag_active, diag(ji,jj,:),                                         &
                  &               diag_fulldepth_active, diag_fulldepth(ji,jj,:,:) )
            END DO
         END DO
         
         ! Loop over each grid point partioning the increments
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi

                  ! Phytoplankton
                  IF ( ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n) > 0.0 ) .AND. &
                     & ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n) > 0.0 ) .AND. &
                     & ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n) > 0.0 ) .AND. &
                     & ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n) > 0.0 ) .AND. &
                     & ( pinc_chltot(ji,jj) /= 0.0 ) ) THEN
                     IF ( ld_chltot ) THEN
                        ! Phytoplankton nitrogen split up based on existing ratios
                        zfrac_p1n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n) / &
                           &        ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n) )
                        zfrac_p2n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n) / &
                           &        ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n) )
                        zfrac_p3n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n) / &
                           &        ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n) )
                        zfrac_p4n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n) / &
                           &        ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n) + &
                           &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n) )
                     ELSE
                        ! Phytoplankton nitrogen split up based on assimilation increments
                        zfrac_p1n = pinc_chldia(ji,jj) / pinc_chltot(ji,jj)
                        zfrac_p2n = pinc_chlnan(ji,jj) / pinc_chltot(ji,jj)
                        zfrac_p3n = pinc_chlpic(ji,jj) / pinc_chltot(ji,jj)
                        zfrac_p4n = pinc_chldin(ji,jj) / pinc_chltot(ji,jj)
                     ENDIF
                     
                     ! Other phytoplankton variables split up based on existing ratios with nitrogen
                     zrat_chl1_p1n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_chl1) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n)
                     zrat_p1c_p1n  = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1c)  / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n)
                     zrat_p1p_p1n  = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1p)  / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n)
                     zrat_p1s_p1n  = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1s)  / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n)
                     zrat_chl2_p2n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_chl2) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n)
                     zrat_p2c_p2n  = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p2c)  / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n)
                     zrat_p2p_p2n  = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p2p)  / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n)
                     zrat_chl3_p3n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_chl3) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n)
                     zrat_p3c_p3n  = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p3c)  / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n)
                     zrat_p3p_p3n  = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p3p)  / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n)
                     zrat_chl4_p4n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_chl4) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n)
                     zrat_p4c_p4n  = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p4c)  / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n)
                     zrat_p4p_p4n  = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p4p)  / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n)
                     
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n)  = outincs(ji,jj,jk,i_tracer(2)) * zfrac_p1n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n)  = outincs(ji,jj,jk,i_tracer(2)) * zfrac_p2n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n)  = outincs(ji,jj,jk,i_tracer(2)) * zfrac_p3n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n)  = outincs(ji,jj,jk,i_tracer(2)) * zfrac_p4n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_chl1) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n) * zrat_chl1_p1n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1c)  = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n) * zrat_p1c_p1n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1p)  = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n) * zrat_p1p_p1n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1s)  = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n) * zrat_p1s_p1n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_chl2) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n) * zrat_chl2_p2n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p2c)  = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n) * zrat_p2c_p2n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p2p)  = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n) * zrat_p2p_p2n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_chl3) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n) * zrat_chl3_p3n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p3c)  = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n) * zrat_p3c_p3n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p3p)  = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n) * zrat_p3p_p3n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_chl4) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n) * zrat_chl4_p4n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p4c)  = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n) * zrat_p4c_p4n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p4p)  = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n) * zrat_p4p_p4n
                  ENDIF

                  ! Zooplankton nitrogen split up based on existing ratios
                  ! Update carbon and phosphorus according to existing ratios
                  IF ( ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z4c) > 0.0 ) .AND. &
                     & ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z5n) > 0.0 ) .AND. &
                     & ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z6n) > 0.0 ) ) THEN
                     zfrac_z4n = ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z4c) * z4qnc ) / &
                        &        ( ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z4c) * z4qnc ) + &
                        &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z5n) + &
                        &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z6n) )
                     zfrac_z5n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z5n) / &
                        &        ( ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z4c) * z4qnc ) + &
                        &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z5n) + &
                        &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z6n) )
                     zfrac_z6n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z6n) / &
                        &        ( ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z4c) * z4qnc ) + &
                        &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z5n) + &
                        &          tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z6n) )
                     zrat_z4c_z4n = 1.0 / z4qnc
                     zrat_z5c_z5n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z5c) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z5n)
                     zrat_z5p_z5n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z5p) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z5n)
                     zrat_z6c_z6n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z6c) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z6n)
                     zrat_z6p_z6n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z6p) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_z6n)
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z5n) = outincs(ji,jj,jk,i_tracer(3)) * zfrac_z5n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z6n) = outincs(ji,jj,jk,i_tracer(3)) * zfrac_z6n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z4c) = outincs(ji,jj,jk,i_tracer(3)) * zfrac_z4n * zrat_z4c_z4n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z5c) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z5n) * zrat_z5c_z5n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z6c) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z6n) * zrat_z6c_z6n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z5p) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z5n) * zrat_z5p_z5n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z6p) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z6n) * zrat_z6p_z6n
                  ENDIF

                  ! Nitrogen nutrient split between nitrate and ammonium based on existing ratios
                  IF ( ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_n3n) > 0.0 ) .AND. &
                     & ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_n4n) > 0.0 ) ) THEN
                     zfrac_n3n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_n3n) / &
                        &        (tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_n3n) + tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_n4n))
                     zfrac_n4n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_n4n) / &
                        &        (tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_n3n) + tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_n4n))
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_n3n) = outincs(ji,jj,jk,i_tracer(1)) * zfrac_n3n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_n4n) = outincs(ji,jj,jk,i_tracer(1)) * zfrac_n4n
                  ENDIF

                  ! Detritus nitrogen split up based on existing ratios
                  ! Update carbon and phosphorus according to existing ratios
                  IF ( ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r4n) > 0.0 ) .AND. &
                     & ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n) > 0.0 ) .AND. &
                     & ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n) > 0.0 ) ) THEN
                     zfrac_r4n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r4n) / &
                        &        (tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r4n) + &
                        &         tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n) + &
                        &         tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n))
                     zfrac_r6n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n) / &
                        &        (tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r4n) + &
                        &         tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n) + &
                        &         tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n))
                     zfrac_r8n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n) / &
                        &        (tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r4n) + &
                        &         tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n) + &
                        &         tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n))
                     zrat_r4c_r4n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r4c) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r4n)
                     zrat_r4p_r4n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r4p) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r4n)
                     zrat_r6c_r6n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r6c) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n)
                     zrat_r6p_r6n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r6p) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n)
                     zrat_r6s_r6n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r6s) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n)
                     zrat_r8c_r8n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r8c) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n)
                     zrat_r8p_r8n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r8p) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n)
                     zrat_r8s_r8n = tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r8s) / tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n)
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r4n) = outincs(ji,jj,jk,i_tracer(1)) * zfrac_r4n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n) = outincs(ji,jj,jk,i_tracer(1)) * zfrac_r6n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n) = outincs(ji,jj,jk,i_tracer(1)) * zfrac_r8n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r4c) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r4n) * zrat_r4c_r4n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r4p) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r4n) * zrat_r4p_r4n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r6c) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n) * zrat_r6c_r6n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r6p) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n) * zrat_r6p_r6n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r6s) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r6n) * zrat_r6s_r6n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r8c) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n) * zrat_r8c_r8n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r8p) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n) * zrat_r8p_r8n
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r8s) = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r8n) * zrat_r8s_r8n
                  ENDIF

                  ! DIC straight from balancing scheme
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_o3c) = outincs(ji,jj,jk,i_tracer(5))

                  ! Alkalinity straight from balancing scheme
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_o3ba) = outincs(ji,jj,jk,i_tracer(6))

                  ! Remove P/R silicon increments from silicate to conserve mass
                  zfrac = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1s) + &
                     &    phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r6s) + &
                     &    phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r8s)
                  IF ( ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_n5s) - zfrac ) > 0.0 ) THEN
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_n5s) = zfrac * (-1.0)
                  ENDIF

                  ! Remove P/Z/R phosphorus increments from phosphate to conserve mass
                  zfrac = phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1p) + &
                     &    phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p2p) + &
                     &    phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p3p) + &
                     &    phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p4p) + &
                     &    phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z5p) + &
                     &    phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_z6p) + &
                     &    phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r4p) + &
                     &    phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r6p) + &
                     &    phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_r8p)
                  IF ( ( tracer_bkg(ji,jj,jk,jp_fabm_m1+jp_fabm_n1p) - zfrac ) > 0.0 ) THEN
                     phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_n1p) = zfrac * (-1.0)
                  ENDIF
                  
               END DO
            END DO
         END DO
      
      ELSE   ! No nitrogen balancing - just update phytoplankton
         
         ! Split up total surface chlorophyll increments
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) > 0.0 ) .AND. &
                  & ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) > 0.0 ) .AND. &
                  & ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) > 0.0 ) .AND. &
                  & ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) > 0.0 ) ) THEN
                  IF ( ld_chltot ) THEN
                     ! Chlorophyll split up based on existing ratios
                     zfrac_chl1 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) /  &
                        &        ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) )
                     zfrac_chl2 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) /  &
                        &        ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) )
                     zfrac_chl3 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) /  &
                        &        ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) )
                     zfrac_chl4 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) /  &
                        &        ( tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) + &
                        &          tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) )
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) = pinc_chltot(ji,jj) * zfrac_chl1
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) = pinc_chltot(ji,jj) * zfrac_chl2
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) = pinc_chltot(ji,jj) * zfrac_chl3
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) = pinc_chltot(ji,jj) * zfrac_chl4
                  ENDIF
                  IF( ld_chldia ) THEN
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) = pinc_chldia(ji,jj)
                  ENDIF
                  IF( ld_chlnan ) THEN
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) = pinc_chlnan(ji,jj)
                  ENDIF
                  IF( ld_chlpic ) THEN
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) = pinc_chlpic(ji,jj)
                  ENDIF
                  IF( ld_chldin ) THEN
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) = pinc_chldin(ji,jj)
                  ENDIF
                  
                  ! Maintain stoichiometric ratios of carbon, nitrogen, phosphorus and silicon
                  IF ( ld_chltot .OR. ld_chldia ) THEN
                     zrat_p1c_chl1 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p1c) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1)
                     zrat_p1n_chl1 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p1n) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1)
                     zrat_p1p_chl1 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p1p) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1)
                     zrat_p1s_chl1 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p1s) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl1)
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p1c) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) * zrat_p1c_chl1
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p1n) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) * zrat_p1n_chl1
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p1p) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) * zrat_p1p_chl1
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p1s) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl1) * zrat_p1s_chl1
                  ENDIF
                  IF ( ld_chltot .OR. ld_chlnan ) THEN
                     zrat_p2c_chl2 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p2c) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl2)
                     zrat_p2n_chl2 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p2n) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl2)
                     zrat_p2p_chl2 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p2p) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl2)
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p2c) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) * zrat_p2c_chl2
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p2n) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) * zrat_p2n_chl2
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p2p) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl2) * zrat_p2p_chl2
                  ENDIF
                  IF ( ld_chltot .OR. ld_chlpic ) THEN
                     zrat_p3c_chl3 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p3c) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl3)
                     zrat_p3n_chl3 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p3n) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl3)
                     zrat_p3p_chl3 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p3p) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl3)
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p3c) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) * zrat_p3c_chl3
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p3n) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) * zrat_p3n_chl3
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p3p) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl3) * zrat_p3p_chl3
                  ENDIF
                  IF ( ld_chltot .OR. ld_chldin ) THEN
                     zrat_p4c_chl4 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p4c) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl4)
                     zrat_p4n_chl4 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p4n) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl4)
                     zrat_p4p_chl4 = tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_p4p) / tracer_bkg(ji,jj,1,jp_fabm_m1+jp_fabm_chl4)
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p4c) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) * zrat_p4c_chl4
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p4n) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) * zrat_p4n_chl4
                     phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p4p) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl4) * zrat_p4p_chl4
                  ENDIF
               ENDIF
            END DO
         END DO
         
         ! Propagate through mixed layer
         DO jj = 1, jpj
            DO ji = 1, jpi
               !
               jkmax = jpk-1
               DO jk = jpk-1, 1, -1
                  IF ( ( pmld(ji,jj) >  gdepw_n(ji,jj,jk)   ) .AND. &
                     & ( pmld(ji,jj) <= gdepw_n(ji,jj,jk+1) ) ) THEN
                     pmld(ji,jj) = gdepw_n(ji,jj,jk+1)
                     jkmax = jk
                  ENDIF
               END DO
               !
               DO jk = 2, jkmax
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_chl1) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl1)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1c)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p1c)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1n)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p1n)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1p)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p1p)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p1s)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p1s)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_chl2) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl2)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p2c)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p2c)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p2n)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p2n)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p2p)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p2p)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_chl3) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl3)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p3c)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p3c)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p3n)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p3n)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p3p)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p3p)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_chl4) = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_chl4)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p4c)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p4c)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p4n)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p4n)
                  phyto2d_balinc(ji,jj,jk,jp_fabm_m1+jp_fabm_p4p)  = phyto2d_balinc(ji,jj,1,jp_fabm_m1+jp_fabm_p4p)
               END DO
               !
            END DO
         END DO

      ENDIF

   END SUBROUTINE asm_phyto2d_bal_ersem

#else
   !!----------------------------------------------------------------------
   !!   Default option : Empty routine
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE asm_phyto2d_bal_ersem(  ld_chltot,                      &
      &                               pinc_chltot,                    &
      &                               ld_chldia,                      &
      &                               pinc_chldia,                    &
      &                               ld_chlnan,                      &
      &                               pinc_chlnan,                    &
      &                               ld_chlpic,                      &
      &                               pinc_chlpic,                    &
      &                               ld_chldin,                      &
      &                               pinc_chldin,                    &
      &                               pincper,                        &
      &                               p_maxchlinc, ld_phytobal, pmld, &
      &                               pgrow_avg_bkg, ploss_avg_bkg,   &
      &                               phyt_avg_bkg, mld_max_bkg,      &
      &                               totalk_bkg,                     &
      &                               tracer_bkg, phyto2d_balinc )
      LOGICAL :: ld_chltot
      REAL    :: pinc_chltot(:,:)
      LOGICAL :: ld_chldia
      REAL    :: pinc_chldia(:,:)
      LOGICAL :: ld_chlnan
      REAL    :: pinc_chlnan(:,:)
      LOGICAL :: ld_chlpic
      REAL    :: pinc_chlpic(:,:)
      LOGICAL :: ld_chldin
      REAL    :: pinc_chldin(:,:)
      REAL    :: pincper
      REAL    :: p_maxchlinc
      LOGICAL :: ld_phytobal
      REAL    :: pmld(:,:)
      REAL    :: pgrow_avg_bkg(:,:)
      REAL    :: ploss_avg_bkg(:,:)
      REAL    :: phyt_avg_bkg(:,:)
      REAL    :: mld_max_bkg(:,:)
      REAL    :: totalk_bkg(:,:,:)
      REAL    :: tracer_bkg(:,:,:,:)
      REAL    :: phyto2d_balinc(:,:,:,:)
      WRITE(*,*) 'asm_phyto2d_bal_ersem: You should not have seen this print! error?'
   END SUBROUTINE asm_phyto2d_bal_ersem
#endif

   !!======================================================================
END MODULE asmphyto2dbal_ersem
